addpath('tools');
clear all;

mu = 0;             % Mean of the distributions
sigma = 1;          % Standard deviation of the distributions
n = 20000;          % Number of samples to use (of each distribution)
x = -6:1e-3:6;      % Points at which to evaluate the two PDFs and the witness function
kernel_sigma = 0.5; % The kernel width to use

% Evaluate the two PDFs at the provided points (used for plotting them)
y_normal = pdf('normal', x, mu, sigma);
y_laplace = exp(-abs(x - mu) / (sigma / sqrt(2))) / (2 * sigma / sqrt(2));

% Sample from the two distributions
normal_sample = normrnd(mu, sigma, n, 1);
laplace_sample = laprnd(mu, sigma, n, 1);

% Compute the MMD and the witness function
mmd = mmd(normal_sample, laplace_sample, @(x1, x2)rbf_kernel(x1, x2, kernel_sigma));
normal_mu = sum(rbf_kernel(x', normal_sample, kernel_sigma), 2) / n;
laplace_mu = sum(rbf_kernel(x', laplace_sample, kernel_sigma), 2) / n;
f_witness = (normal_mu - laplace_mu) / sqrt(mmd);

% Plot the two distrbution PDFs along with the witness function
figure;
hold on;
plot(x, y_normal, 'k', 'LineWidth', 1.5);
plot(x, y_laplace, 'k', 'LineWidth', 1.5, 'LineStyle', '-.');
plot(x, f_witness, 'r', 'LineWidth', 1.5);
title('Probability Densities and Witness Function');
xlabel('$$x$$', 'interpreter', 'latex');
ylabel('Probability Densities and Witness Function');
legend('Normal', 'Laplace', 'Witness Function');
hold off;
% Save the plot
set(gcf, 'PaperPositionMode', 'auto');
print -depsc2 plots/plot_1_normal_laplace_witness_function.eps;
fixPSlinestyle('plots/plot_1_normal_laplace_witness_function.eps', 'plots/plot_1_normal_laplace_witness_function.eps');