addpath('tools');
clear all;

n = 50;      	% Number of samples to use (of each distribution)
x = 0:1e-3:10;  % Points at which to evaluate the witness function
kernel_sigma1 = 2;      % The first kernel width to use
kernel_sigma2 = 0.001;  % The second kernel width to use
alpha = 0.05;                   % The alpha value used for hypothesis testing
numberOfPearsonSamples = 1000;  % The number of samples to draw from the Pearson system when estimating the MMD CDF

%% Generate samples for the different cases we are considering
x1_sample1 = 5 * rand(n, 1);
x2_sample1 = 5 + 5 * rand(n, 1);
x1_sample2 = 10 * rand(n, 1);
x2_sample2 = 10 * rand(n, 1);

%% Compute the MMD and the witness function for the different cases we are considering
mmd1_sigma1 = mmd(x1_sample1, x2_sample1, @(x1, x2)rbf_kernel(x1, x2, kernel_sigma1));
mu1_1_sigma1 = sum(rbf_kernel(x', x1_sample1, kernel_sigma1), 2) / n;
mu2_1_sigma1 = sum(rbf_kernel(x', x2_sample1, kernel_sigma1), 2) / n;
f_witness1_sigma1 = (mu1_1_sigma1 - mu2_1_sigma1) / sqrt(mmd1_sigma1);
[~, ~, ~, test_threshold_1] = mmd_pearson_test(x1_sample1, x2_sample1, @(x1, x2)rbf_kernel(x1, x2, kernel_sigma1), alpha, numberOfPearsonSamples);
case1_accept = mmd1_sigma1 <= test_threshold_1;

mmd1_sigma2 = mmd(x1_sample1, x2_sample1, @(x1, x2)rbf_kernel(x1, x2, kernel_sigma2));
mu1_1_sigma2 = sum(rbf_kernel(x', x1_sample1, kernel_sigma2), 2) / n;
mu2_1_sigma2 = sum(rbf_kernel(x', x2_sample1, kernel_sigma2), 2) / n;
f_witness1_sigma2 = (mu1_1_sigma2 - mu2_1_sigma2) / sqrt(mmd1_sigma2);
[~, ~, ~, test_threshold_2] = mmd_pearson_test(x1_sample1, x2_sample1, @(x1, x2)rbf_kernel(x1, x2, kernel_sigma2), alpha, numberOfPearsonSamples);
case2_accept = mmd1_sigma2 <= test_threshold_2;

mmd2_sigma1 = mmd(x1_sample2, x2_sample2, @(x1, x2)rbf_kernel(x1, x2, kernel_sigma1));
mu1_2_sigma1 = sum(rbf_kernel(x', x1_sample2, kernel_sigma1), 2) / n;
mu2_2_sigma1 = sum(rbf_kernel(x', x2_sample2, kernel_sigma1), 2) / n;
f_witness2_sigma1 = (mu1_2_sigma1 - mu2_2_sigma1) / sqrt(mmd2_sigma1);
[~, ~, ~, test_threshold_3] = mmd_pearson_test(x1_sample2, x2_sample2, @(x1, x2)rbf_kernel(x1, x2, kernel_sigma1), alpha, numberOfPearsonSamples);
case3_accept = mmd2_sigma1 <= test_threshold_3;

mmd2_sigma2 = mmd(x1_sample2, x2_sample2, @(x1, x2)rbf_kernel(x1, x2, kernel_sigma2));
mu1_2_sigma2 = sum(rbf_kernel(x', x1_sample2, kernel_sigma2), 2) / n;
mu2_2_sigma2 = sum(rbf_kernel(x', x2_sample2, kernel_sigma2), 2) / n;
f_witness2_sigma2 = (mu1_2_sigma2 - mu2_2_sigma2) / sqrt(mmd2_sigma2);
[~, ~, ~, test_threshold_4] = mmd_pearson_test(x1_sample2, x2_sample2, @(x1, x2)rbf_kernel(x1, x2, kernel_sigma2), alpha, numberOfPearsonSamples);
case4_accept = mmd2_sigma2 <= test_threshold_4;

%% Plot the two distrbution PDFs along with the witness function
figure('Units', 'normalized', 'Position', [.1 .1 .6 .6]);
subplot(2, 2, 1);
if case1_accept
    title(strcat('$$\sigma=$$', num2str(kernel_sigma1), '$$\;\;\Rightarrow\;\textnormal{MMD }_u^2=$$', num2str(mmd1_sigma1), ' ACCEPTED NULL'), 'interpreter', 'latex');
else
    title(strcat('$$\sigma=$$', num2str(kernel_sigma1), '$$\;\;\Rightarrow\;\textnormal{MMD }_u^2=$$', num2str(mmd1_sigma1), ' REJECTED NULL'), 'interpreter', 'latex');
end
hold on;
scatter(x1_sample1, zeros(size(x1_sample1)), 'Marker', 'o', 'MarkerEdgeColor', 'b', 'LineWidth', 1.5);
scatter(x2_sample1, zeros(size(x2_sample1)), 'Marker', 'o', 'MarkerEdgeColor', [0 .5 0], 'LineWidth', 1.5);
plot(x, f_witness1_sigma1, 'r', 'LineWidth', 1.5);
xlabel('$$x$$', 'interpreter', 'latex');
ylabel('Witness Function');
legend('P Sample', 'Q Sample', 'Witness Function');
hold off;
subplot(2, 2, 2);
if case2_accept
    title(strcat('$$\sigma=$$', num2str(kernel_sigma2), '$$\;\;\Rightarrow\;\textnormal{MMD }_u^2=$$', num2str(mmd1_sigma2), ' ACCEPTED NULL'), 'interpreter', 'latex');
else
    title(strcat('$$\sigma=$$', num2str(kernel_sigma2), '$$\;\;\Rightarrow\;\textnormal{MMD }_u^2=$$', num2str(mmd1_sigma2), ' REJECTED NULL'), 'interpreter', 'latex');
end
hold on;
scatter(x1_sample1, zeros(size(x1_sample1)), 'Marker', 'o', 'MarkerEdgeColor', 'b', 'LineWidth', 1.5);
scatter(x2_sample1, zeros(size(x2_sample1)), 'Marker', 'o', 'MarkerEdgeColor', [0 .5 0], 'LineWidth', 1.5);
plot(x, f_witness1_sigma2, 'r', 'LineWidth', 1.5);
xlabel('$$x$$', 'interpreter', 'latex');
ylabel('Witness Function');
legend('P Sample', 'Q Sample', 'Witness Function');
hold off;
subplot(2, 2, 3);
if case3_accept
    title(strcat('$$\sigma=$$', num2str(kernel_sigma1), '$$\;\;\Rightarrow\;\textnormal{MMD }_u^2=$$', num2str(mmd2_sigma1), ' ACCEPTED NULL'), 'interpreter', 'latex');
else
    title(strcat('$$\sigma=$$', num2str(kernel_sigma1), '$$\;\;\Rightarrow\;\textnormal{MMD }_u^2=$$', num2str(mmd2_sigma1), ' REJECTED NULL'), 'interpreter', 'latex');
end
hold on;
scatter(x1_sample2, zeros(size(x1_sample2)), 'Marker', 'o', 'MarkerEdgeColor', 'b', 'LineWidth', 1.5);
scatter(x2_sample2, zeros(size(x2_sample2)), 'Marker', 'o', 'MarkerEdgeColor', [0 .5 0], 'LineWidth', 1.5);
plot(x, f_witness2_sigma1, 'r', 'LineWidth', 1.5);
xlabel('$$x$$', 'interpreter', 'latex');
ylabel('Witness Function');
legend('P Sample', 'Q Sample', 'Witness Function');
hold off;
subplot(2, 2, 4);
if case4_accept
    title(strcat('$$\sigma=$$', num2str(kernel_sigma2), '$$\;\;\Rightarrow\;\textnormal{MMD }_u^2=$$', num2str(mmd2_sigma2), ' ACCEPTED NULL'), 'interpreter', 'latex');
else
    title(strcat('$$\sigma=$$', num2str(kernel_sigma2), '$$\;\;\Rightarrow\;\textnormal{MMD }_u^2=$$', num2str(mmd2_sigma2), ' REJECTED NULL'), 'interpreter', 'latex');
end
hold on;
scatter(x1_sample2, zeros(size(x1_sample2)), 'Marker', 'o', 'MarkerEdgeColor', 'b', 'LineWidth', 1.5);
scatter(x2_sample2, zeros(size(x2_sample2)), 'Marker', 'o', 'MarkerEdgeColor', [0 .5 0], 'LineWidth', 1.5);
plot(x, f_witness2_sigma2, 'r', 'LineWidth', 1.5);
xlabel('$$x$$', 'interpreter', 'latex');
ylabel('Witness Function');
legend('P Sample', 'Q Sample', 'Witness Function');
hold off;
% Save the plot
set(gcf, 'PaperPositionMode', 'auto');
print -depsc2 plots/plot_2_intuitive_witness_functions.eps;
fixPSlinestyle('plots/plot_2_intuitive_witness_functions.eps', 'plots/plot_2_intuitive_witness_functions.eps');