addpath('tools');
clear all;

d = 2.^(0:1:11);	% Dimensionalities to consider
mu = 0.05*10.^(0:0.1:3);    % Means pairwise Euclidean distances used
sigma = 1;          % That value multipled by the identity matrix gives the covariance matrix of each Gaussian distribution used
n = 250;            % Number of samples to use (of each distribution)
repetitions = 14;   % Number of repetitions of each experiment
alpha = 0.05;       % Hypothesis testing significance level
numberOfPearsonSamples = 100;       % Number of samples to draw from the approximated Pearson CDF for the Pearson MMD test
numberOfBootstrapIterations = 100;  % Number of bootstrap iterations to use for the bootstrap MMD test
numberOfEigenvalues = 2*n-2;        % Number of eigenvalues to use for the spectrum MMD test
numberOfNullSamples = 100;          % Number of samples to draw from the approximated CDF for the spectrum MMD test

%% Run the simulations
type_2_error_uniform_biased = zeros(length(d), 1);
type_2_error_uniform_unbiased = zeros(length(d), 1);
type_2_error_bootstrap = zeros(length(d), 1);
type_2_error_pearson = zeros(length(d), 1);
type_2_error_gamma = zeros(length(d), 1);
type_2_error_spectrum = zeros(length(d), 1);
type_2_error_ttest = zeros(length(d), 1);
dimension_index = 1;
for dimension = d
    current_sigma = sigma * eye(dimension);
    type_2_error_uniform_biased(dimension_index) = 0;
    type_2_error_uniform_unbiased(dimension_index) = 0;
    type_2_error_bootstrap(dimension_index) = 0;
    type_2_error_pearson(dimension_index) = 0;
    type_2_error_gamma(dimension_index) = 0;
    type_2_error_spectrum(dimension_index) = 0;
    type_2_error_ttest(dimension_index) = 0;
    for m = mu
        current_mu = m * ones(1, dimension) / sqrt(dimension);
        repetitions_accepted_uniform_biased = 0;
        repetitions_accepted_uniform_unbiased = 0;
        repetitions_accepted_bootstrap = 0;
        repetitions_accepted_pearson = 0;
        repetitions_accepted_gamma = 0;
        repetitions_accepted_spectrum = 0;
        repetitions_accepted_ttest = 0;
        parfor repetition = 1:repetitions
            x1 = mvnrnd(zeros(1, dimension), current_sigma, n);
            x2 = mvnrnd(current_mu, current_sigma, n);
            kernel_sigma = rbf_kernel_sigma(x1, x2);
            [mmd, threshold] = mmd_uniform_convergence_bound_test(x1, x2, @(x1, x2)rbf_kernel(x1, x2, kernel_sigma), alpha, true);
            if mmd <= threshold
                repetitions_accepted_uniform_biased = repetitions_accepted_uniform_biased + 1;
            end
            [mmd, threshold] = mmd_uniform_convergence_bound_test(x1, x2, @(x1, x2)rbf_kernel(x1, x2, kernel_sigma), alpha, false);
            if mmd <= threshold
                repetitions_accepted_uniform_unbiased = repetitions_accepted_uniform_unbiased + 1;
            end
            [mmd, threshold] = mmd_bootstrap_test(x1, x2, @(x1, x2)rbf_kernel(x1, x2, kernel_sigma), alpha, false, numberOfBootstrapIterations);
            if mmd <= threshold
                repetitions_accepted_bootstrap = repetitions_accepted_bootstrap + 1;
            end
            [mmd, ~, ~, threshold] = mmd_pearson_test(x1, x2, @(x1, x2)rbf_kernel(x1, x2, kernel_sigma), alpha, numberOfPearsonSamples);
            if mmd <= threshold
                repetitions_accepted_pearson = repetitions_accepted_pearson + 1;
            end
            [mmd, threshold] = mmd_gamma_test(x1, x2, @(x1, x2)rbf_kernel(x1, x2, kernel_sigma), alpha);
            if mmd <= threshold
                repetitions_accepted_gamma = repetitions_accepted_gamma + 1;
            end
            [mmd, threshold] = mmd_spectrum_test(x1, x2, @(x1, x2)rbf_kernel(x1, x2, kernel_sigma), alpha, numberOfEigenvalues, numberOfNullSamples);
            if mmd <= threshold
                repetitions_accepted_spectrum = repetitions_accepted_spectrum + 1;
            end
            if hotelling_t_squared_test(x1, x2, alpha) == 1
                repetitions_accepted_ttest = repetitions_accepted_ttest + 1;
            end
        end
        type_2_error_uniform_biased(dimension_index) = type_2_error_uniform_biased(dimension_index) + repetitions_accepted_uniform_biased / repetitions;
        type_2_error_uniform_unbiased(dimension_index) = type_2_error_uniform_unbiased(dimension_index) + repetitions_accepted_uniform_unbiased / repetitions;
        type_2_error_bootstrap(dimension_index) = type_2_error_bootstrap(dimension_index) + repetitions_accepted_bootstrap / repetitions;
        type_2_error_pearson(dimension_index) = type_2_error_pearson(dimension_index) + repetitions_accepted_pearson / repetitions;
        type_2_error_gamma(dimension_index) = type_2_error_gamma(dimension_index) + repetitions_accepted_gamma / repetitions;
        type_2_error_spectrum(dimension_index) = type_2_error_spectrum(dimension_index) + repetitions_accepted_spectrum / repetitions;
        type_2_error_ttest(dimension_index) = type_2_error_ttest(dimension_index) + repetitions_accepted_ttest / repetitions;
    end
    type_2_error_uniform_biased(dimension_index) = type_2_error_uniform_biased(dimension_index) / length(mu);
    type_2_error_uniform_unbiased(dimension_index) = type_2_error_uniform_unbiased(dimension_index) / length(mu);
    type_2_error_bootstrap(dimension_index) = type_2_error_bootstrap(dimension_index) / length(mu);
    type_2_error_pearson(dimension_index) = type_2_error_pearson(dimension_index) / length(mu);
    type_2_error_gamma(dimension_index) = type_2_error_gamma(dimension_index) / length(mu);
    type_2_error_spectrum(dimension_index) = type_2_error_spectrum(dimension_index) / length(mu);
    type_2_error_ttest(dimension_index) = type_2_error_ttest(dimension_index) / length(mu);
    dimension_index = dimension_index + 1;
end

%% Plot the results
figure('Units', 'normalized', 'Position', [.1 .1 .5 .5]);
hold on;
cc = hsv(7);
plot(d, 1 - type_2_error_uniform_biased, 'Color', cc(1,:), 'LineWidth', 1.5, 'Marker', 'o');
plot(d, 1 - type_2_error_uniform_unbiased, 'Color', cc(2,:), 'LineWidth', 1.5, 'Marker', 'o');
plot(d, 1 - type_2_error_bootstrap, 'Color', cc(3,:), 'LineWidth', 1.5, 'Marker', 'o');
plot(d, 1 - type_2_error_pearson, 'Color', cc(4,:), 'LineWidth', 1.5, 'Marker', 'o');
plot(d, 1 - type_2_error_gamma, 'Color', cc(5,:), 'LineWidth', 1.5, 'Marker', 'o');
plot(d, 1 - type_2_error_spectrum, 'Color', cc(6,:), 'LineWidth', 1.5, 'Marker', 'o');
plot(d, 1 - type_2_error_ttest, 'Color', cc(7,:), 'LineWidth', 1.5, 'Marker', 'o');
title('Gaussians with Different Mean and Same Variance/Covariance');
xlabel('Problem Dimensionality');
ylabel('Test Power');
legend('MMD UC Biased', 'MMD UC Unbiased', 'MMD Bootstrap', 'MMD Pearson', 'MMD Gamma', 'MMD Spectrum', 't-Test');
axis([1, d(end), 0, 1]);
set(gca, 'xscale', 'log');
hold off;
set(gcf, 'PaperPositionMode', 'auto');
print -depsc2 plots/plot_5_gaussians_different_mean.eps;
fixPSlinestyle('plots/plot_5_gaussians_different_mean.eps', 'plots/plot_5_gaussians_different_mean.eps');