function accepted = hotelling_t_squared_test(X1, X2, alpha)
% hotelling_t_squared_test tests whether X1 and X2 come from a distribution
% with the same mean, using Hotelling's T-squared two-sample test. alpha
% specifies the significance level of the test. accepted is 1 is the null
% hypothesis is accepted (i.e., X1 and X2 come from a distribution with
% the same mean), and 0 otherwise.

n1 = size(X1, 1);
n2 = size(X2, 1);
p = size(X1, 2);
means_difference = mean(X1) - mean(X2);
% Compute the pooled covariance while shrinking the sample covariances to
% avoid badly conditioned matrices
pooled_covariance = ((n1 - 1) * cov1para(X1) + (n2 - 1) * cov1para(X2)) / (n1 + n2 - 2);
% Compute the T-squared test statistic
test_statistic = means_difference * inv(pooled_covariance) * means_difference';
test_statistic = test_statistic * n1 * n2 / (n1 + n2);
test_statistic = test_statistic * (n1 + n2 - p - 1) / (p * (n1 + n2 - 2));
% Check for acceptance or rejection of the null hypothesis.
if (1 - fcdf(test_statistic, p, n1 + n2 - p - 1)) <= alpha
    accepted = false;
else
    accepted = true;
end

end