function mmd = mmd(x1, x2, kernel, biased)
% mmd computes an MMD estimate for samples x1 and x2, using kernel as the 
% kernel function. That estimate can either be biased (i.e. using 
% V-statistics) or unbiased (i.e. using U-statistics); that will depend on 
% the value of the boolean parameter biased.

if nargin < 4
    biased = false;
end

n = size(x1, 1);
K = kernel(x1, x1);
L = kernel(x2, x2);
KL = kernel(x1, x2);
mmd_sum_terms = (K + L - KL - KL');
if (biased)
    % Compute a biased MMD estimate
    mmd = sum(sum(mmd_sum_terms)) / (n^2);
else
    % Compute the unbiased MMD estimate
    mmd_sum_terms = mmd_sum_terms - diag(diag(mmd_sum_terms));
    mmd = sum(sum(mmd_sum_terms)) / (n*(n-1));
end

end