function [mmd, test_threshold] = ...
    mmd_uniform_convergence_bound_test(x1, x2, kernel, alpha, biased)
% mmd_uniform_convergence_bound_test computes the MMD value for samples x1 
% and x2, using kernel as the kernel function and it then uses the MMD 
% estimate uniform convergence bounds to compute a threshold for performing
% hypothesis testing using parameter alpha as the significance level. The 
% null hypothesis being tested is assuming that x1 and x2 were generated 
% from the same distribution. The biased argument is a boolean variable 
% indicating whether to compute and use a biased or an unbiased MMD 
% estimate.

n = size(x1, 1);
kernel_max = 1; % This value holds for the RBF kernel

% Compute an MMD estimate
K = kernel(x1, x1);
L = kernel(x2, x2);
KL = kernel(x1, x2);
mmd_sum_terms = (K + L - KL - KL');
if (biased)
    % Compute a biased MMD estimate
    mmd = sum(sum(mmd_sum_terms)) / (n^2);
    % Compute the test threshold
    test_threshold = sqrt(2*kernel_max/n) * (1 + sqrt(2 * log(1/alpha)));
else
    % Compute the unbiased MMD estimate
    mmd_sum_terms = mmd_sum_terms - diag(diag(mmd_sum_terms));
    mmd = sum(sum(mmd_sum_terms)) / (n*(n-1));
    % Compute the test threshold
    test_threshold = 4 * kernel_max * sqrt(log(1/alpha)) / sqrt(n);
end

end