function K = rbf_kernel(X1, X2, sigma)
% rbf_kernel generates the kernel matrix computed using the radial basis 
% function kernel for a pair of matrices. X1 is the first matrix, X2 is the
% second matrix and sigma is the kernel width parameter.

% Compute the quadratic terms
X1_Q = sum(X1 .* X1, 2);
X2_Q = sum(X2 .* X2, 2);

% Repeat those terms so that when we add the two matrices we obtain all 
% possible pairwise combinations of them
X1_Q = repmat(X1_Q, 1, size(X2, 1));
X2_Q = repmat(X2_Q', size(X1, 1), 1);

X2_Q = X1_Q + X2_Q - 2 * X1 * X2';  % Compute the exponent
K = exp(-X2_Q / (2 * sigma^2));     % Compute the kernel matrix

end