function rbf_kernel_sigma = rbf_kernel_sigma(x1, x2)
% rbf_kernel_sigma computes the median distance between points in x1 and 
% point in x2 to use as the sigma parameter in the RBF kernel. It can be 
% used to set that parameter automatically if the user does not provide 
% one.

x = [x1; x2];
n = size(x, 1);
x_squared = sum((x .* x), 2);
distances = ...
    repmat(x_squared, 1, n) + repmat(x_squared', n, 1) - 2 * (x * x');
distances = distances - tril(distances);
distances = reshape(distances, n^2, 1);
rbf_kernel_sigma = sqrt(0.5 * median(distances(distances > 0)));

end